import { ObservableVariable, type ObservableVariableOptions } from './ObservableVariable';
/**
 * 可观察 数组
 */
export declare class ObservableArray<T> extends ObservableVariable<readonly T[]> {
    protected readonly _onAdd: Set<(value: T, index: number, oArr: this) => void>;
    protected readonly _onDelete: Set<(value: T, index: number, oArr: this) => void>;
    protected readonly _onUpdate: Set<(newValue: T, oldValue: T, index: number, oArr: this) => void>;
    protected _onBeforeUpdate?: ((newValue: T, oldValue: T, index: number, oArr: this) => T) | undefined;
    /**
     * 数组的长度
     */
    get length(): number;
    set length(v: number);
    /**
     * 获取数组中第一个元素
     */
    get first(): T | undefined;
    /**
     * 获取数组中最后一个元素
     */
    get last(): T | undefined;
    constructor(value?: Iterable<T>, options?: ObservableVariableOptions);
    /**
     * 当变量值发生改变时触发
     */
    on(event: 'set', callback: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => void): void;
    /**
     * 在变量值发生改变之前触发，返回一个新的值用于替换要设置的值
     * 注意：该回调只允许设置一个，重复设置将覆盖之前的回调。该回调不会受 ensureChange 的影响，只要用户设置值就会被触发
     */
    on(event: 'beforeSet', callback: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => readonly T[]): void;
    /**
     * 当更新数组中某个元素的值时触发
     */
    on(event: 'update', callback: (newValue: T, oldValue: T, index: number, oArr: this) => void): void;
    /**
     * 在数组中某个元素的值发生改变之前触发，返回一个新的值用于替换要设置的值
     * 注意：该回调只允许设置一个，重复设置将覆盖之前的回调。该回调不会受 ensureChange 的影响，只要用户设置变量值就会被触发
     */
    on(event: 'beforeUpdate', callback: (newValue: T, oldValue: T, index: number, oArr: this) => T): void;
    /**
     * 当向数组中添加元素时触发
     */
    on(event: 'add', callback: (value: T, index: number, oArr: this) => void): void;
    /**
     * 当删除数组中元素时触发。注意，index 指向的是元素被删除之前在数组中的位置
     */
    on(event: 'delete', callback: (value: T, index: number, oArr: this) => void): void;
    once(event: 'set', callback: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => void): void;
    once(event: 'beforeSet', callback: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => readonly T[]): void;
    once(event: 'update', callback: (newValue: T, oldValue: T, index: number, oArr: this) => void): void;
    once(event: 'beforeUpdate', callback: (newValue: T, oldValue: T, index: number, oArr: this) => T): void;
    once(event: 'add', callback: (value: T, index: number, oArr: this) => void): void;
    once(event: 'delete', callback: (value: T, index: number, oArr: this) => void): void;
    off(event: 'set', callback?: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => void): void;
    off(event: 'beforeSet', callback?: (newValue: readonly T[], oldValue: readonly T[], oArr: this) => readonly T[]): void;
    off(event: 'update', callback?: (newValue: T, oldValue: T, index: number, oArr: this) => void): void;
    off(event: 'beforeUpdate', callback?: (newValue: T, oldValue: T, index: number, oArr: this) => T): void;
    off(event: 'add', callback?: (value: T, index: number, oArr: this) => void): void;
    off(event: 'delete', callback?: (value: T, index: number, oArr: this) => void): void;
    /**
     * 为数组添加或更新一个元素，添加成功将触发 add 事件，更新成功将触发 beforeUpdate 和 update 事件
     * @param index 数组索引(支持反向索引)
     * @param value 要设置的值
     * @returns 设置成功返回 true，设置失败返回 false
     */
    set(index: number, value: T): boolean;
    /**
     * 删除数组中第一个与之匹配的元素，删除成功将会使得数组长度 -1 同时触发 delete 事件
     * @param value 要被删除的值
     * @returns 返回一个布尔值表示删除是否成功
     */
    delete(value: T): boolean;
    /**
     * 删除数组中所有与之匹配的元素，删除成功将触发 delete 事件
     * @param value 要被删除的值
     * @returns 返回被删除元素的个数
     */
    deleteAll(value: T): number;
    /**
     * 从数组中删除最后一个元素，并返回该元素的值，删除成功将触发 delete 事件
     */
    pop(): T | undefined;
    /**
     * 将一个或多个元素添加到数组的末尾，添加成功将触发 add 事件
     * @param items 要添加的元素
     * @returns 返回新数组的长度
     */
    push(...items: readonly T[]): number;
    /**
     * 删除数组中第一个元素，并返回该元素的值，删除成功将触发 delete 事件
     */
    shift(): T | undefined;
    /**
     * 将一个或多个元素添加到数组的开头，添加成功将触发 add 事件
     * @param items 要添加的元素
     * @returns 返回新数组的长度
     */
    unshift(...items: readonly T[]): number;
    /**
     * 从指定位置开始删除数组中剩下的元素。删除成功将触发 delete 事件
     * @param start 从哪开始删除元素
     * @returns 返回被删除的元素
     */
    splice(start: number): T[];
    /**
     * 通过删除或添加元素，改变数组内容。被删除的将触发 delete 事件，可以替换的将触发 update 事件，新添加的将触发 add 事件
     * @param start 从哪开始删除元素
     * @param deleteCount 删除多少个元素
     * @param items 要插入的元素
     * @returns 返回被删除的元素
     */
    splice(start: number, deleteCount: number, ...items: readonly T[]): T[];
    /**
     * 排序数组，排序完成之后将触发 set 和 beforeSet 事件
     * @param compare 大小比较方法
     */
    sort(compare?: (a: T, b: T) => number): this;
    /**
     * 反转数组中元素顺序，反转完成之后将触发 set 和 beforeSet 事件
     */
    reverse(): this;
    /**
     * 将数组中指定部分填充为指定值。如果填充位有值则触发 update 和 beforeUpdate 事件，如果填充位没有值则触发 add 事件
     * @param value 要填充的值
     * @param start 开始位置
     * @param end 结束位置
     */
    fill(value: T, start?: number, end?: number): this;
    /**
     * 浅复制数组的一部分到同一数组中的另一个位置。根据粘贴内容和位置会触发以下几种事件
     * 1. 粘贴一个值到一个已经有值的位置则触发 update 和 beforeUpdate 事件
     * 2. 粘贴一个空位到一个已经有值的位置则触发 delete 事件
     * 3. 粘贴一个值到一个空位则触发 add 事件
     * @param target 插入目标位置
     * @param start 复制起始位置
     * @param end 复制结束位置
     */
    copyWithin(target: number, start: number, end?: number): this;
    /**
     * 读取数组某个位置上的值
     * @param index 位置索引(支持反向索引)
     */
    at(index: number): T | undefined;
    /**
     * at 的别名
     * @param index 位置索引(支持反向索引)
     */
    get(index: number): T | undefined;
    /**
     * 判断数组某个位置上是否有值，注意这个判断针对的是空位而不是 null 或 undefined
     * @param index 位置索引(支持反向索引)
     */
    has(index: number): boolean;
    /**
     * 用于合并两个或多个数组，此方法不会更改现有数组，而是返回一个新数组
     * @param items 要合并的数组
     */
    concat(...items: (T | ConcatArray<T>)[]): T[];
    /**
     * 返回一个 Iterator 对象，该对象包含数组中每个元素的键值对
     */
    entries(): IterableIterator<[number, T]>;
    /**
     * 测试数组的所有元素是否都通过了指定函数的测试
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    every<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): boolean;
    /**
     * 创建一个新数组, 其包含通过指定函数测试的所有元素
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    filter<H>(callback: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): T[];
    /**
     * 返回数组中满足指定函数测试的第一个元素，否则返回 undefined
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    find<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): T | undefined;
    /**
     * 返回数组中满足指定函数测试的最后一个元素，否则返回 undefined
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findLast<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): T | undefined;
    /**
     * 返回数组中满足指定函数测试的第一个元素的索引，否则返回 -1
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findIndex<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): number;
    /**
     * 返回数组中满足指定函数测试的最后一个元素的索引，否则返回 -1
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findLastIndex<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): number;
    /**
     * 扁平化数组，返回一个新的数组来保存扁平化后的结果
     * @param depth 要扁平化的深度，默认1
     */
    flat<D extends number = 1>(depth?: D): FlatArray<readonly T[], D>[];
    /**
     * 等同于 arr.map().flat(1)
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    flatMap<U, H>(callback: (this: H, value: T, index: number, array: T[]) => U | U[], thisArg?: H): U[];
    /**
     * 对数组的每个元素执行一次提供的函数
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    forEach<H>(callback: (this: H, value: T, index: number, array: readonly T[]) => void, thisArg?: H): void;
    /**
     * 判断数组中是否包含一个指定的值
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     * @returns 如果包含则返回 true，否则返回 false
     */
    includes(searchElement: T, fromIndex?: number): boolean;
    /**
     * 返回在数组中找到的第一个元素的索引，如果不存在，则返回 -1
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     */
    indexOf(searchElement: T, fromIndex?: number): number;
    /**
     * 返回在数组中找到的最后一个元素的索引，如果不存在则返回 -1(从数组的后面向前查找，从 fromIndex 处开始)
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     */
    lastIndexOf(searchElement: T, fromIndex?: number): number;
    /**
     * 将一个数组（或一个类数组对象）的所有元素连接成一个字符串并返回这个字符串
     * @param separator 分隔符。默认逗号
     */
    join(separator?: string): string;
    /**
     * 返回一个新的 Array 迭代器，它包含数组中每个索引的键
     */
    keys(): IterableIterator<number>;
    /**
     * 创建一个新数组，其结果是数组中的每个元素在经过回调函数后的值
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    map<U, H>(callback: (this: H, value: T, index: number, array: readonly T[]) => U, thisArg?: H): U[];
    /**
     * 对数组中的每个元素（从左到右）应用一个函数，将其减少为单个值
     */
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T): T;
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T, initialValue: T): T;
    /**
     * 对数组中的每个元素（从右到左）应用一个函数，将其减少为单个值
     */
    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T): T;
    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, array: readonly T[]) => T, initialValue: T): T;
    /**
     * 返回一个从 start 到 end 的数组浅拷贝(原始数组不会被修改)
     * @param start 开始位置。默认开头
     * @param end 结束位置。默认结尾
     */
    slice(start?: number, end?: number): T[];
    /**
     * 判断数组中是否有元素可以通过回调函数的测试
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    some<H>(predicate: (this: H, value: T, index: number, array: readonly T[]) => unknown, thisArg?: H): boolean;
    /**
     * 复制一个数组，然后覆盖掉指定位置的值，如果索引为负数，则从数组末尾替换
     * @param index 要覆盖的位置.
     * @param value 要覆盖的值
     */
    with(index: number, value: T): T[];
    /**
     * 复制一个数组，并反转里面所有的元素
     */
    toReversed(): T[];
    /**
     * 复制一个数组，并排序里面所有的元素
     * @param compareFn 大小比较方法
     */
    toSorted(compareFn?: (a: T, b: T) => number): T[];
    /**
     * 复制一个数组，从指定位置开始，删除指定数量的元素
     * @param start 从什么位置开始删除
     * @param deleteCount 要删除的元素数量。默认删除到结尾
     * @param items 要插入的元素
     */
    toSpliced(start: number, deleteCount?: number, ...items: T[]): T[];
    /**
     * 返回一个字符串表示数组中的元素。数组中的元素将使用各自的 toLocaleString 方法转成字符串，这些字符串将使用一个特定语言环境的字符串（例如一个逗号 ","）隔开
     */
    toLocaleString(): string;
    /**
     * 返回一个字符串，表示数组及其元素。
     */
    toString(): string;
    /**
     * 返回一个 Iterator 对象，该对象包含数组每个索引的值
     */
    values(): IterableIterator<T>;
    /**
     * 返回一个 Iterator 对象，该对象包含数组每个索引的值
     */
    [Symbol.iterator](): IterableIterator<T>;
}
export declare function oArr<T>(value?: Iterable<T>, options?: ObservableVariableOptions): ObservableArray<T>;
