"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservableArray = void 0;
exports.oArr = oArr;
const tslib_1 = require("tslib");
const lodash_isequal_1 = tslib_1.__importDefault(require("lodash.isequal"));
const ObservableVariable_1 = require("./ObservableVariable");
/**
 * JS 数组的最大长度
 */
const JS_ARRAY_MAX_LENGTH = 2 ** 32 - 1;
/**
 * 可观察 数组
 */
class ObservableArray extends ObservableVariable_1.ObservableVariable {
    _onAdd = new Set();
    _onDelete = new Set();
    _onUpdate = new Set();
    _onBeforeUpdate;
    /**
     * 数组的长度
     */
    get length() {
        return this._value.length;
    }
    set length(v) {
        if (v > JS_ARRAY_MAX_LENGTH || v < 0 || !Number.isInteger(v)) {
            throw new RangeError('Invalid array length');
        }
        if (v < this._value.length) {
            const number = this._value.length - v;
            this.splice(this._value.length - number, number);
        }
        else {
            /**
             * this.push(...new Array(v - this._value.length));
             *
             * 当 v > this._value.length 时没有用这种是因为：
             * 1. JS 原生使用 empty 来填充空位，而我这样做用的是 undefined 来填充
             * 2. 通过 length 来拉长数组严格上来说并不算是向数组中添加元素（与在超过数组长度的位置上设置值类似），所以不需要触发 onAdd 事件
             */
            this._value.length = v;
        }
    }
    /**
     * 获取数组中第一个元素
     */
    get first() {
        return this._value[0];
    }
    /**
     * 获取数组中最后一个元素
     */
    get last() {
        return this._value[this._value.length - 1];
    }
    constructor(value = [], options = {}) {
        if (value instanceof ObservableVariable_1.ObservableVariable) {
            return value;
        } // eslint-disable-line no-constructor-return -- 确保变量不会被重复包装
        super(value, options);
        if (!Array.isArray(value)) {
            this._value = Array.from(value);
        }
    }
    on(event, callback) {
        switch (event) {
            case 'update': {
                this._onUpdate.add(callback);
                break;
            }
            case 'beforeUpdate': {
                this._onBeforeUpdate = callback;
                break;
            }
            case 'add': {
                this._onAdd.add(callback);
                break;
            }
            case 'delete': {
                this._onDelete.add(callback);
                break;
            }
            default: {
                super.on(event, callback);
                break;
            }
        }
    }
    once(event, callback) {
        super.once(event, callback);
    }
    off(event, callback) {
        switch (event) {
            case 'update': {
                callback ? this._onUpdate.delete(callback) : this._onUpdate.clear();
                break;
            }
            case 'beforeUpdate': {
                this._onBeforeUpdate = undefined;
                break;
            }
            case 'add': {
                callback ? this._onAdd.delete(callback) : this._onAdd.clear();
                break;
            }
            case 'delete': {
                callback ? this._onDelete.delete(callback) : this._onDelete.clear();
                break;
            }
            default: {
                super.off(event, callback);
                break;
            }
        }
    }
    // #endregion
    // #region 数组修改操作方法
    /**
     * 为数组添加或更新一个元素，添加成功将触发 add 事件，更新成功将触发 beforeUpdate 和 update 事件
     * @param index 数组索引(支持反向索引)
     * @param value 要设置的值
     * @returns 设置成功返回 true，设置失败返回 false
     */
    set(index, value) {
        if (index < 0) {
            index = this._value.length + index;
        }
        if (index < 0) {
            return false;
        }
        if (index in this._value) {
            const oldValue = this._value[index];
            if (this._onBeforeUpdate) {
                value = this._onBeforeUpdate(value, oldValue, index, this);
            }
            if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(value, oldValue) : value === oldValue)) {
                return false;
            }
            this._value[index] = value;
            for (const callback of this._onUpdate) {
                callback(value, oldValue, index, this);
            }
        }
        else {
            this._value[index] = value;
            for (const callback of this._onAdd) {
                callback(value, index, this);
            }
        }
        return true;
    }
    /**
     * 删除数组中第一个与之匹配的元素，删除成功将会使得数组长度 -1 同时触发 delete 事件
     * @param value 要被删除的值
     * @returns 返回一个布尔值表示删除是否成功
     */
    delete(value) {
        const index = this._value.indexOf(value);
        if (index !== -1) {
            this._value.splice(index, 1);
            for (const callback of this._onDelete) {
                callback(value, index, this);
            }
            return true;
        }
        else {
            return false;
        }
    }
    /**
     * 删除数组中所有与之匹配的元素，删除成功将触发 delete 事件
     * @param value 要被删除的值
     * @returns 返回被删除元素的个数
     */
    deleteAll(value) {
        let amount = 0;
        while (this.delete(value)) {
            amount++;
        }
        return amount;
    }
    /**
     * 从数组中删除最后一个元素，并返回该元素的值，删除成功将触发 delete 事件
     */
    pop() {
        if (this._value.length - 1 in this._value) {
            const value = this._value.pop();
            for (const callback of this._onDelete) {
                callback(value, this._value.length, this);
            }
            return value;
        }
        else {
            return this._value.pop();
        }
    }
    /**
     * 将一个或多个元素添加到数组的末尾，添加成功将触发 add 事件
     * @param items 要添加的元素
     * @returns 返回新数组的长度
     */
    push(...items) {
        if (this._onAdd.size > 0) {
            for (const item of items) {
                this._value.push(item);
                for (const callback of this._onAdd) {
                    callback(item, this._value.length - 1, this);
                }
            }
            return this._value.length;
        }
        else {
            return this._value.push(...items);
        }
    }
    /**
     * 删除数组中第一个元素，并返回该元素的值，删除成功将触发 delete 事件
     */
    shift() {
        if (0 in this._value) {
            const value = this._value.shift();
            for (const callback of this._onDelete) {
                callback(value, 0, this);
            }
            return value;
        }
        else {
            return this._value.shift();
        }
    }
    /**
     * 将一个或多个元素添加到数组的开头，添加成功将触发 add 事件
     * @param items 要添加的元素
     * @returns 返回新数组的长度
     */
    unshift(...items) {
        if (this._onAdd.size > 0) {
            for (let index = 0; index < items.length; index++) {
                this._value.splice(index, 0, items[index]);
                for (const callback of this._onAdd) {
                    callback(items[index], index, this);
                }
            }
            return this._value.length;
        }
        else {
            return this._value.unshift(...items);
        }
    }
    splice(start, deleteCount, ...items) {
        if (this._onAdd.size > 0 || this._onUpdate.size > 0 || this._onBeforeUpdate || this._onDelete.size > 0) {
            // 被删除的元素
            const deletedElements = [];
            // 校正开始位置
            start = Math.trunc(start);
            start = start < 0 ? Math.max(this._value.length + start, 0) : Math.min(start, this._value.length);
            // 计算删除数量
            deleteCount = deleteCount === undefined
                ? this._value.length - start
                : deleteCount < 0 ? 0 : Math.min(Math.trunc(deleteCount), this._value.length - start);
            for (let i = 0, j = Math.max(deleteCount, items.length); i < j; i++) {
                let index = start + i;
                if (i < deleteCount && i < items.length) { // 更新元素
                    if (index in this._value) { // 确保不是空位
                        const oldValue = this._value[index];
                        let newValue = items[i];
                        deletedElements.push(oldValue);
                        if (this._onBeforeUpdate) {
                            newValue = this._onBeforeUpdate(newValue, oldValue, index, this);
                        }
                        if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(newValue, oldValue) : newValue === oldValue)) {
                            continue;
                        }
                        this._value[index] = newValue;
                        for (const callback of this._onUpdate) {
                            callback(newValue, oldValue, index, this);
                        }
                    }
                    else {
                        deletedElements.length++;
                        this._value[index] = items[i];
                        for (const callback of this._onAdd) {
                            callback(items[i], index, this);
                        }
                    }
                }
                else if (i < deleteCount) { // 删除元素
                    index = start + items.length;
                    if (index in this._value) { // 确保不是空位
                        const value = this._value.splice(index, 1)[0];
                        for (const callback of this._onDelete) {
                            callback(value, index, this);
                        }
                        deletedElements.push(value);
                    }
                    else {
                        this._value.splice(index, 1);
                        deletedElements.length++;
                    }
                }
                else { // 添加元素
                    this._value.splice(index, 0, items[i]);
                    for (const callback of this._onAdd) {
                        callback(items[i], index, this);
                    }
                }
            }
            return deletedElements;
        }
        else {
            return this._value.splice(start, deleteCount, ...items);
        }
    }
    /**
     * 排序数组，排序完成之后将触发 set 和 beforeSet 事件
     * @param compare 大小比较方法
     */
    sort(compare) {
        if (this._onSet.size > 0 || this._onBeforeSet) {
            this.value = this._value.toSorted(compare);
        }
        else {
            this._value.sort(compare);
        }
        return this;
    }
    /**
     * 反转数组中元素顺序，反转完成之后将触发 set 和 beforeSet 事件
     */
    reverse() {
        if (this._onSet.size > 0 || this._onBeforeSet) {
            this.value = this._value.toReversed();
        }
        else {
            this._value.reverse();
        }
        return this;
    }
    /**
     * 将数组中指定部分填充为指定值。如果填充位有值则触发 update 和 beforeUpdate 事件，如果填充位没有值则触发 add 事件
     * @param value 要填充的值
     * @param start 开始位置
     * @param end 结束位置
     */
    fill(value, start, end) {
        if (this._onAdd.size > 0 || this._onUpdate.size > 0 || this._onBeforeUpdate) {
            start = start === undefined ? 0 : Math.trunc(start);
            end = end === undefined ? this._value.length : Math.trunc(end);
            start = start < 0 ? Math.max(0, this._value.length + start) : Math.min(this._value.length, start);
            end = end < 0 ? Math.max(0, this._value.length + end) : Math.min(this._value.length, end);
            for (; start < end; start++) {
                if (start in this._value) {
                    const oldValue = this._value[start];
                    let newValue = value;
                    if (this._onBeforeUpdate) {
                        newValue = this._onBeforeUpdate(newValue, oldValue, start, this);
                    }
                    if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(newValue, oldValue) : newValue === oldValue)) {
                        continue;
                    }
                    this._value[start] = newValue;
                    for (const callback of this._onUpdate) {
                        callback(newValue, oldValue, start, this);
                    }
                }
                else {
                    this._value[start] = value;
                    for (const callback of this._onAdd) {
                        callback(value, start, this);
                    }
                }
            }
        }
        else {
            this._value.fill(value, start, end);
        }
        return this;
    }
    /**
     * 浅复制数组的一部分到同一数组中的另一个位置。根据粘贴内容和位置会触发以下几种事件
     * 1. 粘贴一个值到一个已经有值的位置则触发 update 和 beforeUpdate 事件
     * 2. 粘贴一个空位到一个已经有值的位置则触发 delete 事件
     * 3. 粘贴一个值到一个空位则触发 add 事件
     * @param target 插入目标位置
     * @param start 复制起始位置
     * @param end 复制结束位置
     */
    copyWithin(target, start, end) {
        if (this._onAdd.size > 0 || this._onUpdate.size > 0 || this._onBeforeUpdate || this._onDelete.size > 0) {
            target = Math.trunc(target);
            start = Math.trunc(start);
            end = end === undefined ? this._value.length : Math.trunc(end);
            target = target < 0 ? Math.max(0, this._value.length + target) : Math.min(this._value.length, target);
            start = start < 0 ? Math.max(0, this._value.length + start) : Math.min(this._value.length, start);
            end = end < 0 ? Math.max(0, this._value.length + end) : Math.min(this._value.length, end);
            const replace = this._value.slice(start, end);
            for (let index = 0; target < this._value.length && index < replace.length; index++, target++) {
                if (index in replace) { // 复制的不是空位
                    if (target in this._value) { // 插入的位置不是空位
                        const oldValue = this._value[target];
                        let newValue = replace[index];
                        if (this._onBeforeUpdate) {
                            newValue = this._onBeforeUpdate(newValue, oldValue, target, this);
                        }
                        if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(newValue, oldValue) : newValue === oldValue)) {
                            continue;
                        }
                        this._value[target] = newValue;
                        for (const callback of this._onUpdate) {
                            callback(newValue, oldValue, target, this);
                        }
                    }
                    else {
                        const newValue = this._value[target] = replace[index];
                        for (const callback of this._onAdd) {
                            callback(newValue, target, this);
                        }
                    }
                }
                else if (target in this._value) {
                    const oldValue = this._value[target];
                    delete this._value[target]; // eslint-disable-line
                    for (const callback of this._onDelete) {
                        callback(oldValue, target, this);
                    }
                }
            }
        }
        else {
            this._value.copyWithin(target, start, end);
        }
        return this;
    }
    // #endregion
    // #region 数组读取操作方法
    /**
     * 读取数组某个位置上的值
     * @param index 位置索引(支持反向索引)
     */
    at(index) {
        return this._value.at(index);
    }
    /**
     * at 的别名
     * @param index 位置索引(支持反向索引)
     */
    get(index) {
        return this._value.at(index);
    }
    /**
     * 判断数组某个位置上是否有值，注意这个判断针对的是空位而不是 null 或 undefined
     * @param index 位置索引(支持反向索引)
     */
    has(index) {
        if (index < 0) {
            index = this._value.length + index;
        }
        return index in this._value;
    }
    /**
     * 用于合并两个或多个数组，此方法不会更改现有数组，而是返回一个新数组
     * @param items 要合并的数组
     */
    concat(...items) {
        return this._value.concat(...items);
    }
    /**
     * 返回一个 Iterator 对象，该对象包含数组中每个元素的键值对
     */
    entries() {
        return this._value.entries();
    }
    /**
     * 测试数组的所有元素是否都通过了指定函数的测试
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    every(predicate, thisArg) {
        return this._value.every(predicate, thisArg);
    }
    /**
     * 创建一个新数组, 其包含通过指定函数测试的所有元素
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    filter(callback, thisArg) {
        return this._value.filter(callback, thisArg);
    }
    /**
     * 返回数组中满足指定函数测试的第一个元素，否则返回 undefined
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    find(predicate, thisArg) {
        return this._value.find(predicate, thisArg);
    }
    /**
     * 返回数组中满足指定函数测试的最后一个元素，否则返回 undefined
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findLast(predicate, thisArg) {
        return this._value.findLast(predicate, thisArg);
    }
    /**
     * 返回数组中满足指定函数测试的第一个元素的索引，否则返回 -1
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findIndex(predicate, thisArg) {
        return this._value.findIndex(predicate, thisArg);
    }
    /**
     * 返回数组中满足指定函数测试的最后一个元素的索引，否则返回 -1
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    findLastIndex(predicate, thisArg) {
        return this._value.findLastIndex(predicate, thisArg);
    }
    /**
     * 扁平化数组，返回一个新的数组来保存扁平化后的结果
     * @param depth 要扁平化的深度，默认1
     */
    flat(depth) {
        return this._value.flat(depth);
    }
    /**
     * 等同于 arr.map().flat(1)
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    flatMap(callback, thisArg) {
        return this._value.flatMap(callback, thisArg);
    }
    /**
     * 对数组的每个元素执行一次提供的函数
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    forEach(callback, thisArg) {
        this._value.forEach(callback, thisArg);
    }
    /**
     * 判断数组中是否包含一个指定的值
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     * @returns 如果包含则返回 true，否则返回 false
     */
    includes(searchElement, fromIndex) {
        return this._value.includes(searchElement, fromIndex);
    }
    /**
     * 返回在数组中找到的第一个元素的索引，如果不存在，则返回 -1
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     */
    indexOf(searchElement, fromIndex) {
        return this._value.indexOf(searchElement, fromIndex);
    }
    /**
     * 返回在数组中找到的最后一个元素的索引，如果不存在则返回 -1(从数组的后面向前查找，从 fromIndex 处开始)
     * @param searchElement 要查找的元素
     * @param fromIndex 从什么位置开始查找
     */
    lastIndexOf(searchElement, fromIndex) {
        return this._value.lastIndexOf(searchElement, fromIndex);
    }
    /**
     * 将一个数组（或一个类数组对象）的所有元素连接成一个字符串并返回这个字符串
     * @param separator 分隔符。默认逗号
     */
    join(separator) {
        return this._value.join(separator);
    }
    /**
     * 返回一个新的 Array 迭代器，它包含数组中每个索引的键
     */
    keys() {
        return this._value.keys();
    }
    /**
     * 创建一个新数组，其结果是数组中的每个元素在经过回调函数后的值
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    map(callback, thisArg) {
        return this._value.map(callback, thisArg);
    }
    reduce(callbackfn, initialValue) {
        return this._value.reduce(callbackfn, initialValue);
    }
    reduceRight(callbackfn, initialValue) {
        return this._value.reduceRight(callbackfn, initialValue);
    }
    /**
     * 返回一个从 start 到 end 的数组浅拷贝(原始数组不会被修改)
     * @param start 开始位置。默认开头
     * @param end 结束位置。默认结尾
     */
    slice(start, end) {
        return this._value.slice(start, end);
    }
    /**
     * 判断数组中是否有元素可以通过回调函数的测试
     * @param predicate 回调函数
     * @param thisArg 回调函数中的 this
     */
    some(predicate, thisArg) {
        return this._value.some(predicate, thisArg);
    }
    /**
     * 复制一个数组，然后覆盖掉指定位置的值，如果索引为负数，则从数组末尾替换
     * @param index 要覆盖的位置.
     * @param value 要覆盖的值
     */
    with(index, value) {
        return this._value.with(index, value);
    }
    /**
     * 复制一个数组，并反转里面所有的元素
     */
    toReversed() {
        return this._value.toReversed();
    }
    /**
     * 复制一个数组，并排序里面所有的元素
     * @param compareFn 大小比较方法
     */
    toSorted(compareFn) {
        return this._value.toSorted(compareFn);
    }
    /**
     * 复制一个数组，从指定位置开始，删除指定数量的元素
     * @param start 从什么位置开始删除
     * @param deleteCount 要删除的元素数量。默认删除到结尾
     * @param items 要插入的元素
     */
    toSpliced(start, deleteCount, ...items) {
        return this._value.toSpliced(start, deleteCount, ...items);
    }
    /**
     * 返回一个字符串表示数组中的元素。数组中的元素将使用各自的 toLocaleString 方法转成字符串，这些字符串将使用一个特定语言环境的字符串（例如一个逗号 ","）隔开
     */
    toLocaleString() {
        return this._value.toLocaleString();
    }
    /**
     * 返回一个字符串，表示数组及其元素。
     */
    toString() {
        return this._value.toString();
    }
    /**
     * 返回一个 Iterator 对象，该对象包含数组每个索引的值
     */
    values() {
        return this._value.values();
    }
    /**
     * 返回一个 Iterator 对象，该对象包含数组每个索引的值
     */
    [Symbol.iterator]() {
        return this._value[Symbol.iterator]();
    }
}
exports.ObservableArray = ObservableArray;
function oArr(value, options) {
    return new ObservableArray(value, options);
}
