import { ObservableVariable, type ObservableVariableOptions } from './ObservableVariable';
/**
 * 可观察 Map
 */
export declare class ObservableMap<K, V> extends ObservableVariable<ReadonlyMap<K, V>> {
    protected readonly _onAdd: Set<(value: V, key: K, oMap: this) => void>;
    protected readonly _onDelete: Set<(value: V, key: K, oMap: this) => void>;
    protected readonly _onUpdate: Set<(newValue: V, oldValue: V, key: K, oMap: this) => void>;
    protected _onBeforeUpdate?: ((newValue: V, oldValue: V, key: K, oMap: this) => V) | undefined;
    /**
     * Map 中元素个数
     */
    get size(): number;
    constructor(value?: Iterable<[K, V]>, options?: ObservableVariableOptions);
    toJSON(): [K, V][] | undefined;
    /**
     * 当变量值发生改变时触发
     */
    on(event: 'set', callback: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => void): void;
    /**
     * 在变量值发生改变之前触发，返回一个新的值用于替换要设置的值
     * 注意：该回调只允许设置一个，重复设置将覆盖之前的回调。该回调不会受 ensureChange 的影响，只要用户设置值就会被触发
     */
    on(event: 'beforeSet', callback: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => ReadonlyMap<K, V>): void;
    /**
     * 当更新 Map 中某个元素的值时触发
     */
    on(event: 'update', callback: (newValue: V, oldValue: V, key: K, oMap: this) => void): void;
    /**
     * 在 Map 中某个元素的值发生改变之前触发，返回一个新的值用于替换要设置的值
     * 注意：该回调只允许设置一个，重复设置将覆盖之前的回调。该回调不会受 ensureChange 的影响，只要用户设置变量值就会被触发
     */
    on(event: 'beforeUpdate', callback: (newValue: V, oldValue: V, key: K, oMap: this) => V): void;
    /**
     * 当向 Map 中添加元素时触发
     */
    on(event: 'add', callback: (value: V, key: K, oMap: this) => void): void;
    /**
     * 当删除 Map 中元素时触发
     */
    on(event: 'delete', callback: (value: V, key: K, oMap: this) => void): void;
    once(event: 'set', callback: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => void): void;
    once(event: 'beforeSet', callback: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => ReadonlyMap<K, V>): void;
    once(event: 'update', callback: (newValue: V, oldValue: V, key: K, oMap: this) => void): void;
    once(event: 'beforeUpdate', callback: (newValue: V, oldValue: V, key: K, oMap: this) => V): void;
    once(event: 'add', callback: (value: V, key: K, oMap: this) => void): void;
    once(event: 'delete', callback: (value: V, key: K, oMap: this) => void): void;
    off(event: 'set', callback?: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => void): void;
    off(event: 'beforeSet', callback?: (newValue: ReadonlyMap<K, V>, oldValue: ReadonlyMap<K, V>, oMap: this) => ReadonlyMap<K, V>): void;
    off(event: 'update', callback?: (newValue: V, oldValue: V, key: K, oMap: this) => void): void;
    off(event: 'beforeUpdate', callback?: (newValue: V, oldValue: V, key: K, oMap: this) => V): void;
    off(event: 'add', callback?: (value: V, key: K, oMap: this) => void): void;
    off(event: 'delete', callback?: (value: V, key: K, oMap: this) => void): void;
    /**
     * 清除 Map 中的所有元素，清除成功将触发 delete 事件
     */
    clear(): void;
    /**
     * 用于删除 Map 中指定的元素，删除成功将触发 delete 事件
     * @param key 要删除的键名
     * @returns 如果 Map 中有这个值就返回 true，否则返回 false
     */
    delete(key: K): boolean;
    /**
     * 为 Map 添加或更新一个元素，添加成功将触发 add 事件，更新将触发 beforeUpdate 和 update 事件
     * @param key 要设置的键名
     * @param value 要设置的值
     */
    set(key: K, value: V): this;
    /**
     * 返回一个 [key, value] 键值对 Iterator 对象，返回的迭代器的迭代顺序与 Map 中元素的插入顺序相同
     */
    entries(): IterableIterator<[K, V]>;
    /**
     * 以插入顺序对 Map 中的每一个键值对执行一次参数中提供的回调函数
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    forEach<H>(callback: (this: H, value: V, key: K, map: ReadonlyMap<K, V>) => void, thisArg?: H): void;
    /**
     * 获取 Map 中指定的元素
     * @param key 要获取元素的键名
     */
    get(key: K): V | undefined;
    /**
     * 返回一个 bool 值，用来表明 Map 中是否存在指定元素
     * @param key 要查询的键名
     */
    has(key: K): boolean;
    /**
     * 返回一个 Iterator 对象，它包含按照顺序插入 Map 对象中每个元素的 key 值
     */
    keys(): IterableIterator<K>;
    /**
     * 返回一个 Iterator 对象，它包含按顺序插入 Map 对象中每个元素的 value 值
     */
    values(): IterableIterator<V>;
    /**
     * 返回一个 [key, value] 键值对 Iterator 对象，返回的迭代器的迭代顺序与 Map 中元素的插入顺序相同
     */
    [Symbol.iterator](): IterableIterator<[K, V]>;
}
export declare function oMap<K, V>(value?: Iterable<[K, V]>, options?: ObservableVariableOptions): ObservableMap<K, V>;
