"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservableMap = void 0;
exports.oMap = oMap;
const tslib_1 = require("tslib");
const lodash_isequal_1 = tslib_1.__importDefault(require("lodash.isequal"));
const ObservableVariable_1 = require("./ObservableVariable");
/**
 * 可观察 Map
 */
class ObservableMap extends ObservableVariable_1.ObservableVariable {
    _onAdd = new Set();
    _onDelete = new Set();
    _onUpdate = new Set();
    _onBeforeUpdate;
    /**
     * Map 中元素个数
     */
    get size() {
        return this._value.size;
    }
    constructor(value = new Map(), options) {
        if (value instanceof ObservableVariable_1.ObservableVariable) {
            return value;
        } // eslint-disable-line no-constructor-return -- 确保变量不会被重复包装
        super(value, options);
        if (!(this._value instanceof Set)) {
            this._value = new Map(value);
        }
    }
    // @ts-expect-error: 由于 Map 的 toJSON 方法只会返回一个空对象，为了让结果有意义，这里将 Map 转换为数组，所以需要强行覆盖父类返回值类型
    toJSON() {
        return this._serializable ? [...this._value] : undefined;
    }
    on(event, callback) {
        switch (event) {
            case 'update': {
                this._onUpdate.add(callback);
                break;
            }
            case 'beforeUpdate': {
                this._onBeforeUpdate = callback;
                break;
            }
            case 'add': {
                this._onAdd.add(callback);
                break;
            }
            case 'delete': {
                this._onDelete.add(callback);
                break;
            }
            default: {
                super.on(event, callback);
                break;
            }
        }
    }
    once(event, callback) {
        super.once(event, callback);
    }
    off(event, callback) {
        switch (event) {
            case 'update': {
                callback ? this._onUpdate.delete(callback) : this._onUpdate.clear();
                break;
            }
            case 'beforeUpdate': {
                this._onBeforeUpdate = undefined;
                break;
            }
            case 'add': {
                callback ? this._onAdd.delete(callback) : this._onAdd.clear();
                break;
            }
            case 'delete': {
                callback ? this._onDelete.delete(callback) : this._onDelete.clear();
                break;
            }
            default: {
                super.off(event, callback);
                break;
            }
        }
    }
    // #endregion
    // #region Map 修改操作方法
    /**
     * 清除 Map 中的所有元素，清除成功将触发 delete 事件
     */
    clear() {
        if (this._onDelete.size > 0) {
            for (const [key, value] of this._value) {
                this._value.delete(key);
                for (const callback of this._onDelete) {
                    callback(value, key, this);
                }
            }
        }
        else {
            this._value.clear();
        }
    }
    /**
     * 用于删除 Map 中指定的元素，删除成功将触发 delete 事件
     * @param key 要删除的键名
     * @returns 如果 Map 中有这个值就返回 true，否则返回 false
     */
    delete(key) {
        if (this._onDelete.size > 0) {
            if (this._value.has(key)) {
                const value = this._value.get(key);
                this._value.delete(key);
                for (const callback of this._onDelete) {
                    callback(value, key, this);
                }
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return this._value.delete(key);
        }
    }
    /**
     * 为 Map 添加或更新一个元素，添加成功将触发 add 事件，更新将触发 beforeUpdate 和 update 事件
     * @param key 要设置的键名
     * @param value 要设置的值
     */
    set(key, value) {
        if (this._value.has(key)) {
            const oldValue = this._value.get(key);
            if (this._onBeforeUpdate) {
                value = this._onBeforeUpdate(value, oldValue, key, this);
            }
            if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(value, oldValue) : value === oldValue)) {
                return this;
            }
            this._value.set(key, value);
            for (const callback of this._onUpdate) {
                callback(value, oldValue, key, this);
            }
        }
        else {
            this._value.set(key, value);
            for (const callback of this._onAdd) {
                callback(value, key, this);
            }
        }
        return this;
    }
    // #endregion
    // #region Map读取操作方法
    /**
     * 返回一个 [key, value] 键值对 Iterator 对象，返回的迭代器的迭代顺序与 Map 中元素的插入顺序相同
     */
    entries() {
        return this._value.entries();
    }
    /**
     * 以插入顺序对 Map 中的每一个键值对执行一次参数中提供的回调函数
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    forEach(callback, thisArg) {
        this._value.forEach(callback, thisArg);
    }
    /**
     * 获取 Map 中指定的元素
     * @param key 要获取元素的键名
     */
    get(key) {
        return this._value.get(key);
    }
    /**
     * 返回一个 bool 值，用来表明 Map 中是否存在指定元素
     * @param key 要查询的键名
     */
    has(key) {
        return this._value.has(key);
    }
    /**
     * 返回一个 Iterator 对象，它包含按照顺序插入 Map 对象中每个元素的 key 值
     */
    keys() {
        return this._value.keys();
    }
    /**
     * 返回一个 Iterator 对象，它包含按顺序插入 Map 对象中每个元素的 value 值
     */
    values() {
        return this._value.values();
    }
    /**
     * 返回一个 [key, value] 键值对 Iterator 对象，返回的迭代器的迭代顺序与 Map 中元素的插入顺序相同
     */
    [Symbol.iterator]() {
        return this._value[Symbol.iterator]();
    }
}
exports.ObservableMap = ObservableMap;
function oMap(value, options) {
    return new ObservableMap(value, options);
}
