import { ObservableVariable, type ObservableVariableOptions } from './ObservableVariable';
/**
 * 可观察 Set
 */
export declare class ObservableSet<T> extends ObservableVariable<ReadonlySet<T>> {
    protected readonly _onAdd: Set<(value: T, oSet: this) => void>;
    protected readonly _onDelete: Set<(value: T, oSet: this) => void>;
    /**
     * Set 中元素个数
     */
    get size(): number;
    constructor(value?: Iterable<T>, options?: ObservableVariableOptions);
    toJSON(): T[] | undefined;
    /**
     * 当变量值发生改变时触发
     */
    on(event: 'set', callback: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => void): void;
    /**
     * 在变量值发生改变之前触发，返回一个新的值用于替换要设置的值
     * 注意：该回调只允许设置一个，重复设置将覆盖之前的回调。该回调不会受 ensureChange 的影响，只要用户设置值就会被触发
     */
    on(event: 'beforeSet', callback: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => ReadonlySet<T>): void;
    /**
     * 当向 Set 中添加元素时触发
     */
    on(event: 'add', callback: (value: T, oSet: this) => void): void;
    /**
     * 当删除 Set 中元素时触发
     */
    on(event: 'delete', callback: (value: T, oSet: this) => void): void;
    once(event: 'set', callback: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => void): void;
    once(event: 'beforeSet', callback: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => ReadonlySet<T>): void;
    once(event: 'add', callback: (value: T, oSet: this) => void): void;
    once(event: 'delete', callback: (value: T, oSet: this) => void): void;
    off(event: 'set', callback?: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => void): void;
    off(event: 'beforeSet', callback?: (newValue: ReadonlySet<T>, oldValue: ReadonlySet<T>, oSet: this) => ReadonlySet<T>): void;
    off(event: 'add', callback?: (value: T, oSet: this) => void): void;
    off(event: 'delete', callback?: (value: T, oSet: this) => void): void;
    /**
     * 清除 Set 中所有元素，清除成功将触发 delete 事件
     */
    clear(): void;
    /**
     * 从 Set 中删除元素，删除成功将触发 delete 事件
     * @param value 要被删除的值
     * @returns 删除成功返回 true，删除失败返回 false
     */
    delete(value: T): boolean;
    /**
     * 向 Set 的末尾添加一个元素，添加成功将触发 add 事件
     * @param value 要被添加的值
     */
    add(value: T): this;
    /**
     * 返回一个包含 [value, value] 键值对的 Iterator 对象，返回的迭代器的迭代顺序与 Set 对象的插入顺序相同
     */
    entries(): IterableIterator<[T, T]>;
    /**
     * 根据 Set 中元素的顺序，对每个元素都执行提供的回调函数一次
     * @param callback 回调函数
     * @param thisArg 回调函数中的 this
     */
    forEach<H>(callback: (this: H, value: T, value2: T, set: ReadonlySet<T>) => void, thisArg?: H): void;
    /**
     * 返回一个布尔值来指示对应的值 value 是否存于 Set 中
     * @param value 要查询的值
     */
    has(value: T): boolean;
    /**
     * 行为与 values 方法完全一致，返回 Set 的元素迭代器
     */
    keys(): IterableIterator<T>;
    /**
     * 返回一个元素迭代器，迭代顺序与 Set 对象的插入顺序相同
     */
    values(): IterableIterator<T>;
    /**
     * 返回一个元素迭代器，迭代顺序与 Set 对象的插入顺序相同
     */
    [Symbol.iterator](): IterableIterator<T>;
    /**
     * 计算两个集合的差集
     * @param other 参与运算的集合
     * @returns 返回一个新的集合，其中包含当前集合中存在但 other 集合中不存在的元素
     */
    difference(other: this | ReadonlySet<T>): Set<T>;
    /**
     * 计算两个集合的交集
     * @param other 参与运算的集合
     * @returns 返回一个新的集合，其中包含两个集合中共同的元素
     */
    intersection(other: this | ReadonlySet<T>): Set<T>;
    /**
     * 计算两个集合的并集
     * @param other 参与运算的集合
     * @returns 返回一个新的集合，其中包含两个集合中所有的元素
     */
    union(other: this | ReadonlySet<T>): Set<T>;
    /**
     * 并集减去交集
     * @param other 参与运算的集合
     */
    symmetricDifference(other: this | ReadonlySet<T>): Set<T>;
    /**
     * 判断两个集合的交集是不是空集（没有共同元素）
     * @param other 参与运算的集合
     * @returns 返回 true 表示没有共同元素，返回 false 表示有共同元素
     */
    isDisjointFrom(other: this | ReadonlySet<T>): boolean;
    /**
     * 判断当前集合是不是 other 集合的子集
     * @param other 参与运算的集合
     * @returns 返回 true 表示是，返回 false 表示不是
     */
    isSubsetOf(other: this | ReadonlySet<T>): boolean;
    /**
     * 判断当前集合是不是 other 集合的父集
     * @param other 参与运算的集合
     * @returns 返回 true 表示是，返回 false 表示不是
     */
    isSupersetOf(other: this | ReadonlySet<T>): boolean;
}
export declare function oSet<T>(value?: Iterable<T>, options?: ObservableVariableOptions): ObservableSet<T>;
