"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservableVariable = void 0;
exports.oVar = oVar;
const tslib_1 = require("tslib");
const lodash_isequal_1 = tslib_1.__importDefault(require("lodash.isequal"));
/**
 * 可观察变量
 */
class ObservableVariable {
    _serializable;
    _ensureChange;
    _deepCompare;
    _onSet = new Set();
    _onBeforeSet;
    _value;
    /**
     * 获取变量值，设置值将触发 beforeSet 和 set 事件
     */
    get value() {
        return this._value;
    }
    set value(v) {
        if (this._onBeforeSet) {
            v = this._onBeforeSet(v, this._value, this);
        }
        if (this._ensureChange && (this._deepCompare ? (0, lodash_isequal_1.default)(v, this._value) : v === this._value)) {
            return;
        }
        const oldValue = this._value;
        this._value = v;
        for (const callback of this._onSet) {
            callback(v, oldValue, this);
        }
    }
    /**
     * 注意：如果要继承 ObservableVariable，那么一定要添加下面一段代码
     * ```
     * if(value instanceof ObservableVariable) { return value }   // 确保变量不会被重复包装
     * super(value, options);
     * ```
     * @param value 要被观察的变量
     * @param options 配置参数
     */
    constructor(value, options = {}) {
        // 确保变量不会被重复包装
        if (value instanceof ObservableVariable) {
            /**
             * 如果造中使用了 return，那么不管是父类还是子类的原型都将不会附加到该对象上
             * 构造函数也就退化成了普通方法，子类构造中的 this 等于 return 的返回值
             */
            return value; // eslint-disable-line no-constructor-return
        }
        const { serializable = true, ensureChange = true, deepCompare = false } = options;
        this._serializable = serializable;
        this._ensureChange = ensureChange;
        this._deepCompare = deepCompare;
        this._value = value;
    }
    toJSON() {
        return this._serializable ? this._value : undefined;
    }
    on(event, callback) {
        switch (event) {
            case 'set':
                this._onSet.add(callback);
                break;
            case 'beforeSet':
                this._onBeforeSet = callback;
                break;
        }
    }
    once(event, callback) {
        const tempCallback = (...args) => { this.off(event, tempCallback); return callback(...args); };
        this.on(event, tempCallback);
    }
    off(event, callback) {
        switch (event) {
            case 'set': {
                callback ? this._onSet.delete(callback) : this._onSet.clear();
                break;
            }
            case 'beforeSet': {
                this._onBeforeSet = undefined;
                break;
            }
        }
    }
}
exports.ObservableVariable = ObservableVariable;
function oVar(value, options) {
    return new ObservableVariable(value, options);
}
